// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML object to serve as an exit dialog
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3

Item {

    id: atmModelApplyItem
    anchors.fill: parent

    // Rectangle to draw the shaded background
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: "black"
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        anchors.fill: atmModelApplyColumnLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        color: "white"
        border.width: 1
        border.color: "gray"
        radius: coreObj.defaultLabelHeight * 0.5
    }

    // Column layout with all interface components
    ColumnLayout {

        id: atmModelApplyColumnLayout
        anchors.centerIn: parent

        CustomLabel {
            id: descriptionLabel
            text: "The values in the selected atmospheric\nmodel at ground altitude (total pressure,\ntemperature, and partial pressures) will\nbe overwritten into the horizontal path\ncalculation variables.\n\nWould you like to continue?\n"
            font.pointSize: coreObj.defaultFontPointSize + 2 * (1 - coreObj.fontScalingRatio)
        }

        RowLayout {
            id: noYesRowLayout
            Layout.alignment: Qt.AlignHCenter

            CustomDialogButton {
                id: noButton
                text: qsTr("No")
                implicitWidth: atmModelApplyColumnLayout.width * 0.48
                implicitHeight: ((Qt.platform.os === 'winrt') || (Qt.platform.os === 'winphone')) ? coreObj.defaultLabelHeight * 2.4 : coreObj.defaultLabelHeight * 2.7
                onClicked: {
                    atmModelApplyItem.destroy();
                }
            }

            CustomDialogButton {
                id: yesButton
                text: qsTr("Yes")
                implicitWidth: atmModelApplyColumnLayout.width * 0.48
                implicitHeight: ((Qt.platform.os === 'winrt') || (Qt.platform.os === 'winphone')) ? coreObj.defaultLabelHeight * 2.4 : coreObj.defaultLabelHeight * 2.7
                onClicked: {
                    atmListViewModel.assignNewValues();  // Reading in the atmospehric model values for the horizontal path for the current atmospheric model
                    molListViewModel.updatePartPress();  // Need to also update the values displayed in the molecules list (partial pressures)
                    remoteControlQml.broadcastCommand("ATMMODEL", 0)
                    atmModelApplyItem.destroy();
                }
            }
        }
    }
}
