// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The bytran logo is distributed under the Creative Commons 3.0 Attribution License: https://creativecommons.org/licenses/by/3.0/us/
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


// Component is used to authenticate with the HITRAN server (it is currently possible to access the HITRAN server without authentication but this open access functionality may be terminated in the future)
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3

Item {

    id: authenticateItem
    anchors.fill: parent

    // Rectangle to shade the background
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: 'black'
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        anchors.fill: mainAuthenticateColumnLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        color: "white"
        border.width: 1
        border.color: 'gray'
        radius: coreObj.defaultLabelHeight * 0.5
    }

    // Exit button
    CustomButton {
        id: exitButton
        anchors.top: authenticateItem.top
        anchors.right: authenticateItem.right
        anchors.margins: coreObj.defaultLabelHeight * (0.1)
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("X")
        onClicked: authenticateItem.destroy()
    }

    CustomLabel {
        id: widthLabel
        text: qsTr("The optical pathlength units and value")
        visible: false
        enabled: false
    }

    // Column layout to hold all the interface items
    ColumnLayout {
        id: mainAuthenticateColumnLayout
        anchors.centerIn: parent

        CustomLabel {
            text: qsTr("Log in to the HITRANonline (hitran.org) server")
        }

        // Radio buttons to select anonymous access vs. password protected one
        RowLayout {
            id: anonymousAuthenticateRowLayout
            anchors.bottomMargin:mainAuthenticateColumnLayout.spacing
            Layout.alignment: Qt.AlignHCenter

            CustomLabel {
                id: useSensorTagNoAuxLabel
                text: qsTr("Anonymous")
                color: 'gray'
                MouseArea {
                    anchors.fill: parent
                    onClicked: authenticate.authenticate = false
                }
            }
            CustomSwitch {
                id: useAuthenticationSwitch
                padding: 0
                checked: authenticate.authenticate
                onCheckedChanged: {
                    authenticate.authenticate = checked
                }
            }
            CustomLabel {
                id: useSensorTagYesAuxLabel
                text: qsTr("Authenticate")
                color: 'red'
                MouseArea {
                    anchors.fill: parent
                    onClicked: authenticate.authenticate = true
                }
            }
        }

        ColumnLayout {
            id: mainColumnLayout            

            TabBar {
                id: loginOptionsBar
                width: parent.width
                bottomPadding: coreObj.defaultLabelHeight * (0.15)
                currentIndex: authenticate.authenticateType
                onCurrentIndexChanged: {
                    authenticate.authenticateType = currentIndex
                }

                CustomTabButton {
                    width: commentLabel.width * 0.5
                    text: qsTr("Email & Password")
                }
                CustomTabButton {
                    width: commentLabel.width * 0.49
                    text: qsTr("HAPI token")
                }
            }

            StackLayout {
                id: displayedParameterStackLayout
                Layout.fillWidth: true
                currentIndex: loginOptionsBar.currentIndex
                enabled: useAuthenticationSwitch.checked

                // Email and password grid layout
                GridLayout {
                    id: emailAndPasswordGridLayout
                    columns: 2
                    rows: 2

                    // Email address
                    CustomLabel {
                        id: emailAddressLabel
                        anchors.right: emailAddressTextField.left
                        anchors.rightMargin: coreObj.defaultLabelHeight * 0.3
                        horizontalAlignment: Text.AlignRight
                        text: qsTr("Email\naddress:")
                        Layout.column: 0
                        Layout.row: 0
                    }
                    CustomTextField {
                        id: emailAddressTextField
                        Layout.fillWidth: true
                        Layout.column: 1
                        Layout.row: 0
                        text: authenticate.email
                        onEditingFinished: {
                            authenticate.email = text
                        }
                    }

                    // Password
                    CustomLabel {
                        id: passwordLabel
                        anchors.right: passwordTextField.left
                        anchors.rightMargin: coreObj.defaultLabelHeight * 0.3
                        // Had to hide this control to resize the StackLayout to visible contents
                        visible: loginOptionsBar.currentIndex == 0 ? true : false
                        horizontalAlignment: Text.AlignRight
                        text: qsTr("Password:")
                        Layout.column: 0
                        Layout.row: 1
                    }
                    CustomTextField {
                        id: passwordTextField
                        // Had to hide this control to resize the StackLayout to visible contents
                        visible: loginOptionsBar.currentIndex == 0 ? true : false
                        Layout.fillWidth: true
                        Layout.column: 1
                        Layout.row: 1
                        text: authenticate.password
                        onEditingFinished: {
                            authenticate.password = text
                        }
                    }
                }

                // HAPI token
                RowLayout {
                    id: hapiTokenRowLayout

                    CustomLabel {
                        id: hapiTokenLabel
                        anchors.right: hapiTokenTextField.left
                        anchors.rightMargin: coreObj.defaultLabelHeight * 0.3
                        horizontalAlignment: Text.AlignRight
                        text: qsTr("HAPI\ntoken:")
                    }
                    CustomTextField {
                        id: hapiTokenTextField
                        Layout.fillWidth: true
                        text: authenticate.hapiToken
                        onEditingFinished: {
                            authenticate.hapiToken = text
                        }
                    }
                }
            }

            Rectangle {
                id: separatorRectangle
                Layout.fillWidth: true
                height: 1
                color: '#D3D3D3'
            }

            // Bottom controls row layout
            RowLayout {
                id: bottomRowLayout
                enabled: useAuthenticationSwitch.checked

                CustomDialogButtonWithBackgroundSmall {
                    id: settingsButton
                    Layout.fillWidth: true
                    text: qsTr("Settings")
                    // Opening another window with the authentication settings parameters
                    onClicked: {
                        var component = Qt.createComponent("AuthenticateSettings.qml")
                        if (component.status === Component.Ready)
                        {
                            authentitateSettingsWindow = component.createObject(authenticateItem)
                        }
                        authenticateSettingsWindow.show()
                    }
                }
                CustomDialogButtonWithBackgroundSmall {
                    id: testButton
                    Layout.fillWidth: true
                    text: qsTr("Test authentication")
                }
            }
        }
    }
}
