// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The bytran logo is distributed under the Creative Commons 3.0 Attribution License: https://creativecommons.org/licenses/by/3.0/us/
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


// This control is used to specify the HITRAN database authentication settings
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3

Item {

    id: authenticateSettingsItem
    anchors.fill: parent

    // Rectangle to shade the background
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: 'black'
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        anchors.fill: mainAuthenticateSettingsColumnLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        color: "white"
        border.width: 1
        border.color: 'gray'
        radius: coreObj.defaultLabelHeight * 0.5
    }

    // Exit button
    CustomButton {
        id: exitButton
        anchors.top: authenticateSettingsItem.top
        anchors.right: authenticateSettingsItem.right
        anchors.margins: coreObj.defaultLabelHeight * (0.1)
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("X")
        onClicked: authenticateSettingsItem.destroy()
    }

    CustomLabel {
        id: widthLabel
        text: qsTr("Store the login information")
        visible: false
        enabled: false
    }

    // Column layout to hold all the interface items
    ColumnLayout {
        // At rotation the buttons are not positioned exactly the way they should be (fix this)
        id: mainAuthenticateSettingsColumnLayout
        anchors.centerIn: parent
        spacing: 5
        CustomLabel {
            id: commentLabel
            Layout.fillWidth: true
            text: qsTr("Store the login information")
        }

        Rectangle {
            id: separatorRectangle
            Layout.fillWidth: true
            height: 1
            color: '#D3D3D3'
        }

        CustomCheckBox {
            id: storeEmailCheckBox
            text: qsTr("Store email")
            checked: authenticate.storeEmail
            onCheckedChanged: authenticate.storeEmail = checked
        }

        CustomCheckBox {
            id: storePasswordCheckBox
            text: qsTr("Store password")
            checked: authenticate.storePassword
            onCheckedChanged: authenticate.storePassword = checked
        }

        CustomCheckBox {
            id: storeHapiTokenCheckBox
            text: qsTr("Store HAPI token")
            checked: authenticate.storeHapiToken
            onCheckedChanged: authenticate.storeHapiToken = checked
        }
    }
}
