// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML component to connect bytran to external devices via serial port (RS-232) or Bluetooth for device integration purposes
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3
import QtQuick.Window 2.3

Item {

    id: connectItem
    anchors.fill: parent

    // Reizing the window when loading is complete
    Component.onCompleted: resizeComponent()

    // This Javascript function is used to resize the connect item window
    // Resizing of form not done very nicely, need to figure out a better way in the future
    function resizeComponent() {
        if (remoteControlQml.communicationMode === 0) { mainConnectColumnLayout.height = commentLabel.height + bluetoothButton.height + serialPortButton.height + wiFiEthernetButton.height + 3 * mainConnectColumnLayout.spacing; }
        if (remoteControlQml.communicationMode === 1) { mainConnectColumnLayout.height = commentLabel.height + connectComboBox.height + bluetoothServerLabel1.height + bluetoothServerUuidLabel.height + bluetoothServerLabel2.height + bluetoothBroadcastManualInputRowLayout.height + separator0Rect.height + 7.0 * mainConnectColumnLayout.spacing; }
        if (remoteControlQml.communicationMode === 2) { mainConnectColumnLayout.height = rs232GridLayout.height + connectComboBox.height + commentLabel.height + 2 * mainConnectColumnLayout.spacing; }
        if (remoteControlQml.communicationMode === 3) { mainConnectColumnLayout.height = commentLabel.height + connectComboBox.height + ipCommentAuxLabel.height + bytranDeviceIpAuxLabel.height + bytranDeviceIpAddressLabel.height + remoteDeviceIpRowLayout.height + ipBroadcastManualInputRowLayout.height + 6.5 * mainConnectColumnLayout.spacing; }
    }

    // When the screen is rotated adjusting the position and orientation of the button used to display information about the communicaion protocol
    onHeightChanged:
    {
        // Need to see if can come up with a better solution than just assigning the item height like that
        if (Screen.primaryOrientation === Qt.PortraitOrientation)
        {
            protocolBorderRect.rotation = 0
            protocolButton.rotation = 0
            protocolButton.anchors.topMargin = 0
            protocolButton.anchors.leftMargin = 0
        }
        else
        {
            protocolBorderRect.rotation = 270
            protocolButton.rotation = 270
            protocolButton.anchors.topMargin = - 0.5 * protocolButton.width - 0.5 * protocolButton.height - coreObj.defaultLabelHeight * 0.85
            protocolButton.anchors.leftMargin = - 0.5 * protocolButton.width - 0.5 * protocolButton.height - coreObj.defaultLabelHeight * 0.85
        }

        // Resizing the connect window
        resizeComponent()
    }

    // Rectangle to shade the background
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: 'black'
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        id: formBorderRect
        anchors.fill: mainConnectColumnLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        color: "white"
        border.width: 1
        border.color: 'gray'
        radius: coreObj.defaultLabelHeight * 0.5
    }

    // Exit button
    CustomButton {
        id: exitButton
        anchors.top: connectItem.top
        anchors.right: connectItem.right
        anchors.margins: coreObj.defaultLabelHeight * 0.1
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("X")
        onClicked: {
            connectItem.destroy();
        }
    }

    Rectangle {
        id: protocolBorderRect
        anchors.fill: protocolButton
        anchors.margins: coreObj.defaultLabelHeight * (-0.85) + 1
        color: "white"
        border.width: 1
        border.color: 'white'
        radius: coreObj.defaultLabelHeight * 0.5
        visible: remoteControlQml.communicationMode === 0 ? false : true
    }
    CustomDialogButtonWithBackground {
        id: protocolButton
        anchors.top: formBorderRect.bottom
        anchors.left:  mainConnectColumnLayout.left
        text: qsTr("  How to use?  ")
        visible: remoteControlQml.communicationMode === 0 ? false : true
        onClicked:
        {
            if (remoteControlQml.communicationMode === 0) { remoteControlQml.openUrl("http://www.bytran.org/protocol.htm") }
            else if (remoteControlQml.communicationMode === 1) { remoteControlQml.openUrl("http://www.bytran.org/bluetooth.htm") }
            else if (remoteControlQml.communicationMode === 2) { remoteControlQml.openUrl("http://www.bytran.org/serialport.htm") }
            else if (remoteControlQml.communicationMode === 3) { remoteControlQml.openUrl("http://www.bytran.org/websockets.htm") }
        }
        onRotationChanged: { bluetoothColumnLayout.update(); }
    }

    CustomLabel {
        id: widthLabel
        text: qsTr("Connection setup to interface with bytran")
        visible: false
        enabled: false
    }

    // Column layout to hold all the interface items
    ColumnLayout {
        id: mainConnectColumnLayout
        anchors.centerIn: parent
        anchors.horizontalCenter: connectItem.horizontalCenter
        anchors.top: (Screen.primaryOrientation === Qt.PortraitOrientation) ? exitButton.bottom : connectItem.top
        anchors.topMargin: coreObj.defaultLabelHeight * 1.2
        anchors.bottom: connectItem.bottom
        anchors.bottomMargin: (Screen.primaryOrientation === Qt.PortraitOrientation) ? coreObj.defaultLabelHeight * 1.2 + protocolButton.height : 1.2 + protocolButton.height
        onHeightChanged: resizeComponent()

        CustomLabel {
            id: commentLabel
            anchors.top: mainConnectColumnLayout.top
            text: remoteControlQml.communicationMode == 0 ? qsTr("Connection setup to interface with bytran\nfor remote control and retrieval of results") : qsTr("Connection setup to interface with bytran")
            onHeightChanged: resizeComponent()
        }

        // Connection method (if enabled)
        CustomComboBox {
            id: connectComboBox
            anchors.top: commentLabel.bottom
            anchors.topMargin: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'android') || (Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? commentLabel.height * 0.4 : 7
            anchors.bottomMargin: commentLabel.height * 0.8
            Layout.fillWidth: true
            model: [ "Disable connectivity", "Bluetooth", "RS-232 (serial port)", "WiFi / Ethernet"]
            currentIndex: remoteControlQml.communicationMode
            onCurrentIndexChanged: {
                remoteControlQml.communicationMode = currentIndex;
                resizeComponent();
            }
            visible: remoteControlQml.communicationMode == 0 ? false : true
        }
        CustomDialogButtonWithBackground {
            id: bluetoothButton
            Layout.fillWidth: true
            text: qsTr("Bluetooth")
            visible: remoteControlQml.communicationMode == 0 ? true : false
            onClicked: {
                remoteControlQml.communicationMode = 1;
                resizeComponent();
            }
        }
        CustomDialogButtonWithBackground {
            id: serialPortButton
            Layout.fillWidth: true
            text: qsTr("RS-232 (serial port)")
            visible: remoteControlQml.communicationMode == 0 ? true : false
            onClicked: {
                remoteControlQml.communicationMode = 2;
                resizeComponent();
            }
        }
        CustomDialogButtonWithBackground {
            id: wiFiEthernetButton
            Layout.fillWidth: true
            text: qsTr("WiFi / Ethernet")
            visible: remoteControlQml.communicationMode == 0 ? true : false
            onClicked: {
                remoteControlQml.communicationMode = 3;
                resizeComponent();
            }
        }

        ColumnLayout {
            id: ipColumnLayout
            visible: remoteControlQml.communicationMode == 3 ? true : false

            CustomLabel {
                id: ipCommentAuxLabel
                anchors.left: ipColumnLayout.left
                text: qsTr("bytran is running a WebSockets server.\nTo control bytran remotely over the internet,\ninitiate a WebSockets client connection to\nbytran from another device and then use the\nbytran communication protocol.")
            }

            RowLayout {
                id: ipInfoRowLayout
                anchors.right: ipColumnLayout.right
                anchors.left: ipColumnLayout.left

                ColumnLayout {
                    id: ipInfoColumnLayout
                    anchors.left: ipInfoRowLayout.left

                    CustomLabel {
                        id: bytranDeviceIpAuxLabel
                        padding: 0
                        anchors.left: ipInfoColumnLayout.left
                        text: qsTr("Your active IP address(es): ")
                    }
                    CustomLabel {
                        id: bytranDeviceIpAddressLabel
                        padding: 0
                        anchors.top: bytranDeviceIpAuxLabel.bottom
                        anchors.left: ipInfoColumnLayout.left
                        color: 'red'
                        text: remoteControlQml.bytranDeviceIpAddress
                    }
                    RowLayout {
                        id: remoteDeviceIpRowLayout
                        anchors.top: bytranDeviceIpAddressLabel.bottom
                        anchors.left: ipInfoColumnLayout.left
                        anchors.topMargin: mainConnectColumnLayout.spacing
                        CustomLabel {
                            id: remoteDeviceIpAuxLabel
                            padding: 0
                            text: (remoteControlQml.remoteDeviceIpAddress === "") ? qsTr("Client disconnected.") : qsTr("Client:")
                        }
                        CustomLabel {
                            id: remoteDeviceIpAddressLabel
                            padding: 0
                            text: (remoteControlQml.remoteDeviceIpAddress === "") ? qsTr('') : remoteControlQml.remoteDeviceIpAddress
                        }
                    }
                }

                Rectangle {
                    id: separator2Rect
                    Layout.fillHeight: true
                    width: 1
                    color: '#C0C0C0'
                }
                Rectangle {
                    id: separator3Rect
                    Layout.fillHeight: true
                    width: 1
                    color: 'white'
                }

                ColumnLayout {
                    id: portNumberColumnLayout
                    anchors.top: ipInfoRowLayout.top

                    CustomLabel {
                        id: portNumberAuxLabel
                        padding: 0
                        text: qsTr("port number:")
                    }
                    CustomDialogButtonWithBackgroundSmall {
                        id: portNumberButton
                        padding: 0
                        implicitWidth: portNumberAuxLabel.width
                        implicitHeight: portNumberAuxLabel.height * 2.2
                        text: remoteControlQml.portNumber
                        onClicked: {
                            var component = Qt.createComponent("PortNumber.qml")
                            if (component.status === Component.Ready)
                            {
                                portNumberWindow = component.createObject(connectItem)
                            }
                            portNumberWindow.show()
                        }
                    }
                }
            }

            Rectangle {
                id: separator1Rect
                anchors.top: ipInfoRowLayout.bottom
                anchors.topMargin: mainConnectColumnLayout.spacing
                Layout.fillWidth: true
                Layout.column: 0
                Layout.row: 3
                height: 1
                color: '#C0C0C0'
            }

            RowLayout {
                id: ipBroadcastManualInputRowLayout
                anchors.top: separator1Rect.bottom
                anchors.topMargin: mainConnectColumnLayout.spacing

                CustomLabel {
                    id:  ipBroadcastManualInputAuxLabel
                    text: qsTr("Broadcast manual input: ")
                    MouseArea {
                        anchors.fill: parent
                        onClicked: ipBroadcastManualInputSwitch.checked = !ipBroadcastManualInputSwitch.checked
                    }
                }
                CustomLabel {
                    id: ipBroadcastManualInputNoAuxLabel
                    text: qsTr("No")
                    color: 'gray'
                    MouseArea {
                        anchors.fill: parent
                        onClicked: remoteControlQml.broadcastManualInput = false
                    }
                }
                CustomSwitch {
                    id: ipBroadcastManualInputSwitch
                    padding: 0
                    checked: remoteControlQml.broadcastManualInput
                    onCheckedChanged: remoteControlQml.broadcastManualInput = checked
                }
                CustomLabel {
                    id: ipBroadcastManualInputYesAuxLabel
                    text: qsTr("Yes")
                    color: 'red'
                    MouseArea {
                        anchors.fill: parent
                        onClicked: remoteControlQml.broadcastManualInput = true
                    }
                }
            }
        }

        // Bluetooth item to set the bluetooth connections
        ColumnLayout {
            id: bluetoothColumnLayout
            anchors.top: connectComboBox.bottom
            anchors.bottom:  mainConnectColumnLayout.bottom
            anchors.topMargin: mainConnectColumnLayout.spacing
            visible: remoteControlQml.communicationMode == 1 ? true : false

            CustomLabel {
                id: bluetoothServerLabel1
                text: qsTr("bytran is running a Bluetooth server. To\nconnect to bytran initiate a client connection\nusing the Serial Port Profile (SPP) with the\nservice UUID:")
            }
            CustomLabel {
                id: bluetoothServerUuidLabel
                text: remoteControlQml.connectedString
                color: 'red'
            }
            CustomLabel {
                id: bluetoothServerLabel2
                text: qsTr("Client bluetooth connection for devices like\nHC-06 which only support slave Bluetooth\nmode is currently not unavailable.")
            }

            Rectangle {
                id: separator0Rect
                anchors.top: bluetoothServerLabel2.bottom
                anchors.topMargin: mainConnectColumnLayout.spacing * 1.5
                Layout.fillWidth: true
                Layout.column: 0
                Layout.row: 3
                height: 1
                color: '#C0C0C0'
            }

            RowLayout {
                id: bluetoothBroadcastManualInputRowLayout
                anchors.bottom: bluetoothColumnLayout.bottom
                anchors.topMargin: mainConnectColumnLayout.spacing

                CustomLabel {
                    id: bluetoothBroadcastManualInputAuxLabel
                    text: qsTr("Broadcast manual input: ")
                    MouseArea {
                        anchors.fill: parent
                        onClicked: bluetoothBroadcastManualInputSwitch.checked = !bluetoothBroadcastManualInputSwitch.checked
                    }
                }
                CustomLabel {
                    id: bluetoothBroadcastManualInputNoAuxLabel
                    text: qsTr("No")
                    color: 'gray'
                    MouseArea {
                        anchors.fill: parent
                        onClicked: remoteControlQml.broadcastManualInput = false
                    }
                }
                CustomSwitch {
                    id: bluetoothBroadcastManualInputSwitch
                    padding: 0
                    checked: remoteControlQml.broadcastManualInput
                    onCheckedChanged: remoteControlQml.broadcastManualInput = checked
                }
                CustomLabel {
                    id: bluetoothBroadcastManualInputYesAuxLabel
                    text: qsTr("Yes")
                    color: 'red'
                    MouseArea {
                        anchors.fill: parent
                        onClicked: remoteControlQml.broadcastManualInput = true
                    }
                }
            }
        }

        // RS-232 item to set the COM port for communication
        GridLayout {
            id: rs232GridLayout
            visible: remoteControlQml.communicationMode == 2 ? true : false

            ColumnLayout {
                id: rs232portColumnLayout
                Layout.column: 0
                Layout.row: 0

                // Selected port
                CustomLabel {
                    id: rs232PortLabel
                    text: qsTr("Selected port:")
                }
                CustomComboBox {
                    id: rs232PortComboBox
                    implicitWidth: widthLabel.width * 0.54
                    implicitHeight: rs232DataBitsSpinBox.height
                    model: serialPortListModel
                    currentIndex: serialPortQml.portIndex
                    onCurrentIndexChanged: serialPortQml.portIndex = currentIndex
                }
            }

            ColumnLayout {
                id: rs232BaudRateColumnLayout
                Layout.column: 0
                Layout.row: 1

                // Baud rate
                CustomLabel {
                    id: rs232BaudRateLabel
                    text: qsTr("Baud rate:")
                }
                CustomComboBox {
                    id: rs232BaudRateComboBox
                    implicitWidth: widthLabel.width * 0.54
                    implicitHeight: rs232DataBitsSpinBox.height
                    model: ["1200", "2400", "4800", "9600", "19200", "38400", "57600", "115200"]
                    currentIndex: serialPortQml.baudRate
                    onCurrentIndexChanged: serialPortQml.baudRate = currentIndex
                }
            }

            ColumnLayout {
                id: dataBitsColumnLayout
                Layout.column: 1
                Layout.row: 0

                // Data bits
                CustomLabel {
                    id: rs232DataBitsLabel
                    Layout.column: 0
                    Layout.row: 0
                    text: qsTr("Data bits:")
                }
                SpinBox {
                    id: rs232DataBitsSpinBox
                    from: 5
                    to: 8
                    value: serialPortQml.dataBits
                    onValueChanged: serialPortQml.dataBits = value
                    implicitWidth: widthLabel.width * 0.54
                }
            }

            ColumnLayout {
                id: stopBitsColumnLayout
                Layout.column: 1
                Layout.row: 1

                // Stop bits
                CustomLabel {
                    id: rs232StopBitsLabel
                    Layout.column: 0
                    Layout.row: 2
                    text: qsTr("Stop bits:")
                }
                SpinBox {
                    id: rs232StopBitsSpinBox
                    from: 1
                    to: 2
                    value: serialPortQml.stopBits
                    onValueChanged: serialPortQml.stopBits = value
                    implicitWidth: widthLabel.width * 0.54
                }
            }

            ColumnLayout {
                id: parityColumnLayout
                Layout.column: 0
                Layout.row: 2

                // Parity
                CustomLabel {
                    id: rs232ParityLabel
                    Layout.column: 0
                    Layout.row: 1
                    text: qsTr("Parity:")
                }
                CustomComboBox {
                    id: rs232ParityComboBox
                    implicitWidth: widthLabel.width * 0.54
                    implicitHeight: rs232DataBitsSpinBox.height
                    model: ["None" , "Odd", "Even", "Space", "Mark"]
                    currentIndex: serialPortQml.parity
                    onCurrentIndexChanged: serialPortQml.parity = currentIndex
                }
            }

            ColumnLayout {
                id: boradcastManualInputColumnLayout
                anchors.horizontalCenter: stopBitsColumnLayout.horizontalCenter
                Layout.column: 1
                Layout.row: 2

                // Broadening
                RowLayout {
                    id: switchRowLayout
                    anchors.left: boradcastManualInputColumnLayout.left
                    anchors.right: boradcastManualInputColumnLayout.right

                    CustomLabel {
                        id: onRowLabel
                        anchors.right: rs232boradcastManualInputSwitch.left
                        text: qsTr("No")
                        color: 'gray'
                        MouseArea {
                            anchors.fill: parent
                            onClicked: remoteControlQml.broadcastManualInput = false
                        }
                    }
                    CustomSwitch {
                        id: rs232boradcastManualInputSwitch
                        anchors.centerIn: parent
                        padding: 0
                        checked: remoteControlQml.broadcastManualInput
                        onCheckedChanged: remoteControlQml.broadcastManualInput = checked
                    }
                    CustomLabel {
                        id: offRowLabel
                        anchors.left: rs232boradcastManualInputSwitch.right
                        text: qsTr("Yes");
                        color: 'red'
                        MouseArea {
                            anchors.fill: parent
                            onClicked: remoteControlQml.broadcastManualInput = true
                        }
                    }
                }

                // Comment label
                RowLayout {
                    anchors.left: boradcastManualInputColumnLayout.left
                    anchors.right: boradcastManualInputColumnLayout.right
                    CustomLabel {
                        id:  boradcastManualInputLabel
                        anchors.centerIn: parent
                        text: qsTr("  Broadcast\nmanual input")
                        MouseArea {
                            anchors.fill: parent
                            onClicked: rs232boradcastManualInputSwitch.checked = !rs232boradcastManualInputSwitch.checked
                        }
                    }
                }
            }
        }        
    }
}
