// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// Object containing the main menu
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3
import QtQuick.Window 2.3

Item {

    id: menuItem
    anchors.fill: parent
    opacity: 1.0

    // Making sure the menu is centered relative to the parent widget (QML window)
    anchors.horizontalCenter: parent.horizontalCenter;
    anchors.verticalCenter: parent.verticalCenter;

    // Setting the default enabled and disabled buttons once the form has finished loading
    Component.onCompleted:
    {
        windowNumText.text = "menu page 1 of 2"

        wavelengthsButton.enabled = true
        pathlengthButton.enabled = true
        totalPressureButton.enabled = true
        temperatureButton.enabled = true
        yAxisButton.enabled = true
        atmModelButton.enabled = true
        lineOptionsButton.enabled = false
        instrFunctButton.enabled = false
        hitranDownloadOptionsButton.enabled = false
        shareResultsButton.enabled = false
        sensorsButton.enabled = false
        connectButton.enabled = false

        wavelengthsButton.visible = true
        pathlengthButton.visible = true
        totalPressureButton.visible = true
        temperatureButton.visible = true
        yAxisButton.visible = true
        atmModelButton.visible = true
        lineOptionsButton.visible = false
        instrFunctButton.visible = false
        hitranDownloadOptionsButton.visible = false
        shareResultsButton.visible = false
        sensorsButton.visible = false
        connectButton.visible = false
    }

    // Used to rotate the menu and display show the button used to open the information about the program
    onHeightChanged:
    {        
        if (Screen.primaryOrientation === Qt.PortraitOrientation) {
            mainGridLayout.flow = GridLayout.TopToBottom
        }
        else {
            mainGridLayout.flow = GridLayout.LeftToRight
        }
    }

    // Rectangle to shade the background
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: 'black'
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the border around items
    Rectangle {
        id: menuRect
        anchors.fill: mainGridLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        color: "white"
        border.width: 1
        border.color: 'gray'
        radius: coreObj.defaultLabelHeight * 0.5
    }

    CustomButton {
        id: exitButton
        anchors.top: menuItem.top
        anchors.right: menuItem.right
        anchors.margins: coreObj.defaultLabelHeight * (0.1)
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("X")
        onClicked: menuItem.destroy()
    }

    // Displaying information about the program (only displayed in the sandscape mode)
    CustomButton {
        id: aboutButton
        anchors.top: menuItem.top
        anchors.left: menuItem.left
        anchors.margins: coreObj.defaultLabelHeight * (0.1)
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("?")

        // Opening another window with information about the application
        onClicked: {
            var component = Qt.createComponent("About.qml")
            if (component.status === Component.Ready)
            {
                aboutWindow = component.createObject(menuItem)
            }
            aboutWindow.show()
        }
    }

    RowLayout {
        id: bottomRowLayout
        Layout.fillWidth: true
        anchors.bottom: menuItem.bottom
        anchors.right: menuItem.right
        anchors.left: menuItem.left
        anchors.margins: coreObj.defaultLabelHeight * (0.1)
        opacity: 1.0

        CustomButton {
            id: prevButton
            anchors.left: bottomRowLayout.left
            anchors.margins: 1
            implicitWidth: 1.5 * prevButton.height
            text: qsTr("<<")

            // Showing the first window of the menu
            onClicked: {

                windowNumText.text = "menu page 1 of 2"

                wavelengthsButton.enabled = true
                pathlengthButton.enabled = true
                totalPressureButton.enabled = true
                temperatureButton.enabled = true
                yAxisButton.enabled = true
                atmModelButton.enabled = true
                lineOptionsButton.enabled = false
                instrFunctButton.enabled = false
                hitranDownloadOptionsButton.enabled = false
                shareResultsButton.enabled = false
                sensorsButton.enabled = false
                connectButton.enabled = false

                wavelengthsButton.visible = true
                pathlengthButton.visible = true
                totalPressureButton.visible = true
                temperatureButton.visible = true
                yAxisButton.visible = true
                atmModelButton.visible = true
                lineOptionsButton.visible = false
                instrFunctButton.visible = false
                hitranDownloadOptionsButton.visible = false
                shareResultsButton.visible = false
                sensorsButton.visible = false
                connectButton.visible = false
            }
        }

        Rectangle {
            id: windowNumRectangle
            anchors.left: prevButton.right
            anchors.right: nextButton.left
            Layout.fillHeight: true
            border.width: 0
            border.color: "white"
            color: 'transparent'

            Text {
                id: windowNumText
                anchors.centerIn: parent                
                text: "menu page 1 of 2"
                font.bold: true
                horizontalAlignment: Text.AlignCenter
                color: 'white'
            }
        }

        CustomButton {
            id: nextButton
            anchors.right: bottomRowLayout.right
            anchors.margins: 1
            implicitWidth:  1.5 * nextButton.height
            text: qsTr(">>")            

            // Showing the second window of the menu
            onClicked: {

                windowNumText.text = "menu page 2 of 2"

                wavelengthsButton.enabled = false
                pathlengthButton.enabled = false
                totalPressureButton.enabled = false
                temperatureButton.enabled = false
                yAxisButton.enabled = false
                atmModelButton.enabled = false
                lineOptionsButton.enabled = true
                instrFunctButton.enabled = true
                hitranDownloadOptionsButton.enabled = true
                shareResultsButton.enabled = true
                sensorsButton.enabled = true
                connectButton.enabled = true

                wavelengthsButton.visible = false
                pathlengthButton.visible = false
                totalPressureButton.visible = false
                temperatureButton.visible = false
                yAxisButton.visible = false
                atmModelButton.visible = false
                lineOptionsButton.visible = true
                instrFunctButton.visible = true
                hitranDownloadOptionsButton.visible = true
                shareResultsButton.visible = true
                sensorsButton.visible = true
                connectButton.visible = true
            }
        }
    }

    GridLayout {
        // At rotation the buttons are not positioned exactly the way they should be (fix this)
        id: mainGridLayout
        anchors.centerIn: parent        
        rowSpacing: 5
        rows: 10
        columns: 2

        // Changing the wavelength range
        CustomButton {
            id: wavelengthsButton
            implicitWidth: atmModelButton.width
            text: qsTr("  Wavelengths  ")            

            // Opening another window with wavelength parameters
            onClicked: {
                var component = Qt.createComponent("Wavelengths.qml")
                if (component.status === Component.Ready)
                {
                    wavelengthWindow = component.createObject(menuItem)
                }
                wavelengthWindow.show()
            }
        }

        // Changing the pathlength selected
        CustomButton {
            id: pathlengthButton
            implicitWidth: atmModelButton.width
            text: qsTr("  Pathlength  ")

            // Opening another window with pathlength parameters
            onClicked: {
                var component = Qt.createComponent("Pathlength.qml")
                if (component.status === Component.Ready)
                {
                    pathlengthWindow = component.createObject(menuItem)
                }
                pathlengthWindow.show()
            }
        }

        // Changing the total pressure value
        CustomButton {
            id: totalPressureButton
            implicitWidth: atmModelButton.width
            text: qsTr("  Total pressure  ")

            // Opening another window with total pressure parameters
            onClicked: {
                var component = Qt.createComponent("TotalPressure.qml")
                if (component.status === Component.Ready)
                {
                    totalPressureWindow = component.createObject(menuItem)
                }
                totalPressureWindow.show()
            }
        }

        // Changing the temperature value
        CustomButton {
            id: temperatureButton
            implicitWidth: atmModelButton.width
            text: qsTr("  Temperature  ")

            // Opening another window with temperature parameters
            onClicked: {
                var component = Qt.createComponent("Temperature.qml")
                if (component.status === Component.Ready)
                {
                    temperatureWindow = component.createObject(menuItem)
                }
                temperatureWindow.show()
            }
        }

        // Changing the vertical axis range
        CustomButton {
            id: yAxisButton
            text: qsTr("Vertical axis")
            implicitWidth: atmModelButton.width

            // Opening another window with vertical axis parameters
            onClicked: {
                var component = Qt.createComponent("YAxis.qml")
                if (component.status === Component.Ready)
                {
                    yAxisWindow = component.createObject(menuItem)
                }
                yAxisWindow.show()
            }
        }

        // Changing the atmospheric model selected
        CustomButton {
            id: atmModelButton            
            text: qsTr("  Atmosphere model  ")

            // Opening another window with atmospehric model parameters
            onClicked: {
                var component = Qt.createComponent("AtmosphericModel.qml")
                if (component.status === Component.Ready)
                {
                    atmModelWindow = component.createObject(menuItem)
                }
                atmModelWindow.show()
            }
        }

        // Setting the instrument function
        CustomButton {
            id: instrFunctButton
            text: qsTr("Instrument funct.")
            implicitWidth: atmModelButton.width

            // Opening another window with instrument function parameters
            onClicked: {
                var component = Qt.createComponent("InstrumentFunction.qml")
                if (component.status === Component.Ready)
                {
                    instrFunctWindow = component.createObject(menuItem)
                }
                instrFunctWindow.show()
            }
        }

        // Changing line options
        CustomButton {
            id: lineOptionsButton
            text: qsTr("  Line options  ")
            implicitWidth: atmModelButton.width

            // Opening another window with line options parameters
            onClicked: {
                var component = Qt.createComponent("LineOptions.qml")
                if (component.status === Component.Ready)
                {
                    lineOptionsWindow = component.createObject(menuItem)
                }
                lineOptionsWindow.show()
            }
        }

        // Setting the instrument function
        CustomButton {
            id: hitranDownloadOptionsButton
            text: qsTr("HITRAN options")
            implicitWidth: atmModelButton.width

            // Opening another window with HITRAN options parameters
            onClicked: {
                var component = Qt.createComponent("HitranDownloadOptions.qml")
                if (component.status === Component.Ready)
                {
                    hitranDownloadOptionsWindow = component.createObject(menuItem)
                }
                hitranDownloadOptionsWindow.show()
            }
        }

        // Emailing or sharing through other channel the results of the calculations (the plot, data or both)
        CustomButton {
            id: shareResultsButton
            text: qsTr("Share results")
            implicitWidth: atmModelButton.width

            // Opening another window with share results options
            onClicked: {
                var component = Qt.createComponent("ShareResultsChoice.qml")
                if (component.status === Component.Ready)
                {
                    shareResultsChoiceWindow = component.createObject(menuItem)
                }
                shareResultsChoiceWindow.show()
            }
        }       

        // Viewing the sensors information and setting up sampling frequency
        CustomButton {
            id: sensorsButton
            text: qsTr("Sensors")
            implicitWidth: atmModelButton.width

            // Opening another window with sensors parameters
            onClicked: {
                var component = Qt.createComponent("Sensors.qml")
                if (component.status === Component.Ready)
                {
                    sensorsWindow = component.createObject(menuItem)
                }
                sensorsWindow.show()
            }
        }

        // Displaying information about the program (only displayed in the sandscape mode)
        CustomButton {
            id: connectButton
            text: qsTr("Connect")
            implicitWidth: atmModelButton.width

            // Opening another window with external device connection options (RS-232, and Bluetooth)
            onClicked: {
                serialPortListModel.updateDisplayed()  // This is done to update the list of available COM ports (in case additional devices have been added)
                var component = Qt.createComponent("Connect.qml")
                if (component.status === Component.Ready)
                {
                    connectWindow = component.createObject(menuItem)
                }
                connectWindow.show()
            }
        }
    }
}
