// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML component used to change the wavenumber resolution (as number of data points or absolute range)
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3

Item {

    id: wavelengthResolutionItem
    anchors.fill: parent

    // Rectangle to shade the background
    Rectangle {
        id: exitButtonRect
        anchors.fill: parent
        border.width: 1
        border.color: 'black'
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        anchors.fill: mainWavelengthResolutionColumnLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        color: "white"
        border.width: 1
        border.color: 'gray'
        radius: coreObj.defaultLabelHeight * 0.5
    }

    CustomLabel {
        id: widthLabel
        text: "The wavelength resolution settings       "
        visible: false
        enabled: false
    }

    // Exit button
    CustomButton {
        id: exitButton
        anchors.top: wavelengthResolutionItem.top
        anchors.right: wavelengthResolutionItem.right
        anchors.margins: coreObj.defaultLabelHeight * (0.1)
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("X")
        onClicked: {
            wavelengthResolutionItem.destroy();
        }
    }

    // Grid layout with the main form components
    ColumnLayout {

        // At rotation the buttons are not positioned exactly the way they should be (fix this)
        id: mainWavelengthResolutionColumnLayout
        anchors.centerIn: parent
        spacing: 5
        CustomLabel {
            id: commentLabel
            text: {      if (wlQml.wnResolMode === 0)  { qsTr("The resolution is specified by its absolute\nvalue in current wavelength units") }
                    else if (wlQml.wnResolMode === 1)  { qsTr("The coarser of the absolute resolution and\nthat calculated from the number of points") }
                    else if (wlQml.wnResolMode === 2)  { qsTr("The resolution is specified by the number\nof points over current wavelength span") } }
        }

        TabBar {
            id: resolutionModeBar
            width: parent.width
            bottomPadding: coreObj.defaultLabelHeight * (0.15)
            currentIndex: wlQml.wnResolMode
            onCurrentIndexChanged: {
                wlQml.wnResolMode = currentIndex
                remoteControlQml.broadcastCommand("WNRESMODE", 0)
            }

            CustomTabButton {
                width: mainWavelengthResolutionColumnLayout.width * 0.33
                text: qsTr("Absolute\nresolution")
            }
            CustomTabButton {
                width: mainWavelengthResolutionColumnLayout.width * 0.34
                text: qsTr("Coarser\nof the two")
            }
            CustomTabButton {
                width: mainWavelengthResolutionColumnLayout.width * 0.33
                text: qsTr("Data\npoints")
            }
        }

        GridLayout {

            CustomLabel {

                id: calculatedValuesAuxLabel
                Layout.column: 0
                Layout.row: 0
                anchors.right: resolutionWavelengthTextField.left
                horizontalAlignment: Text.AlignRight
                rightPadding: coreObj.defaultLabelHeight * 0.5
                text: qsTr("Active values:")
            }

            GridLayout {

                Layout.column: 1
                Layout.row: 0

                // Emailing the results of the calculations (the plot, data or both)
                CustomLabel {
                    id: calcResolLabel
                    anchors.right: resolutionWavelengthTextField.left
                    horizontalAlignment: Text.AlignRight
                    Layout.column: 0
                    Layout.row: 0
                    text: qsTr("Resolution:  ")
                }
                CustomLabel {
                    id: calcResolValueLabel
                    Layout.column: 1
                    Layout.row: 0
                    color: 'red'
                    text: wlQml.calculatedWlResolCurrUnits
                }

                // Emailing the results of the calculations (the plot, data or both)
                CustomLabel {
                    id: calcNumPointsLabel
                    anchors.right: resolutionWavelengthTextField.left
                    horizontalAlignment: Text.AlignRight
                    rightPadding: coreObj.defaultLabelHeight * 0.5
                    Layout.column: 0
                    Layout.row: 1
                    text: qsTr("Data points:")
                }
                CustomLabel {
                    id: calcNumPointsValueLabel
                    Layout.column: 1
                    Layout.row: 1
                    color: 'red'
                    text: wlQml.calculatedNumPoints
                }
            }

            // Changing the resolution as a value in current units
            CustomLabel {
                id: resolutionLabel
                anchors.right: resolutionTextField.left
                Layout.column: 0
                Layout.row: 1
                text: qsTr("Resolution:  ")
            }
            CustomTextField {
                id: resolutionTextField
                Layout.column: 1
                Layout.row: 1
                implicitWidth: widthLabel.width -  resolutionLabel.width
                enabled: ((wlQml.wnResolMode === 0) || (wlQml.wnResolMode === 1)) ? true : false
                text: wlQml.resolWavelength
                onEditingFinished: {
                    wlQml.resolWavelength = text
                    remoteControlQml.broadcastCommand("WNRES", 0)
                }
            }

            // Changing the wavelength number of points
            CustomLabel {
                id: numPointsLabel
                anchors.right: resolutionWavelengthTextField.left
                Layout.column: 0
                Layout.row: 2
                text: qsTr("Data points:  ")
            }
            CustomTextField {
                id: resolutionWavelengthTextField
                Layout.column: 1
                Layout.row: 2
                implicitWidth: widthLabel.width -  resolutionLabel.width
                enabled: ((wlQml.wnResolMode === 2) || (wlQml.wnResolMode === 1)) ? true : false
                text: wlQml.numPoints
                onEditingFinished: {
                    wlQml.numPoints = text
                    remoteControlQml.broadcastCommand("NUMWNPTS", 0)
                }
            }            
        }

        // Wavelength units
        CustomComboBox {
            id: unitsComboBox
            Layout.fillWidth: true
            model: [ "Wavelength, nm",  "Wavelength, micron", "Wavenumber, cm-1", "Frequency, THz" ]  // 0 - Wavelength, nm;  1 - Wavelength, micron, 2 - Wavenumber, cm-1;  3 - Frequency, Hz;  ( 4 - Attenuation, Db try to add later, this is not dB but something involving Db, need to check )
            currentIndex: wlQml.units
            onCurrentIndexChanged: wlQml.units = currentIndex
        }
    }
}
