// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML component used to change the range of calculation wavelengths
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3

Item {

    id: wavelengthsItem
    anchors.fill: parent

    // Rectangle to shade the background
    Rectangle {
        id: exitButtonRect
        anchors.fill: parent
        border.width: 1
        border.color: 'black'
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        anchors.fill: mainWavelengthColumnLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        color: "white"
        border.width: 1
        border.color: 'gray'
        radius: coreObj.defaultLabelHeight * 0.5
    }

    CustomLabel {
        id: widthLabel
        text: "The wavelength spectral range and units"
        visible: false
        enabled: false
    }

    // Exit button
    CustomButton {
        id: exitButton
        anchors.top: wavelengthsItem.top
        anchors.right: wavelengthsItem.right
        anchors.margins: coreObj.defaultLabelHeight * (0.1)
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("X")
        onClicked: {
            specResultsQml.updateXAxis();
            specResultsQml.updatePlotAllPreserveScale();
            wavelengthsItem.destroy();
        }
    }

    // Grid layout with the main form components
    ColumnLayout {

        // At rotation the buttons are not positioned exactly the way they should be (fix this)
        id: mainWavelengthColumnLayout
        anchors.centerIn: parent
        spacing: 5
        CustomLabel {
            id: commentLabel
            text: qsTr("The wavelength spectral range and units")
        }
        GridLayout {

            // Changing the total pressure value
            CustomLabel {
                id: startWavelengthLabel
                anchors.right: startWavelengthTextField.left
                Layout.column: 0
                Layout.row: 0
                text: qsTr("         Start:  ")
            }
            CustomTextField {
                id: startWavelengthTextField
                Layout.column: 1
                Layout.row: 0
                anchors.right: mainWavelengthColumnLayout.right
                implicitWidth: widthLabel.width - startWavelengthLabel.width
                text: wlQml.startWavelength
                onEditingFinished: {
                    wlQml.startWavelength = text
                    remoteControlQml.broadcastCommand("SWAVE", 0)
                }
            }

            // Changing the wavelength range
            CustomLabel {
                id: endWavelengthLabel
                anchors.right: endWavelengthTextField.left
                Layout.column: 0
                Layout.row: 1
                text: qsTr("           End:  ")
            }
            CustomTextField {
                id: endWavelengthTextField
                Layout.column: 1
                Layout.row: 1
                implicitWidth: widthLabel.width - startWavelengthLabel.width
                text: wlQml.endWavelength
                onEditingFinished: {
                    wlQml.endWavelength = text
                    remoteControlQml.broadcastCommand("EWAVE", 0)
                }
            }
        }

        CustomDialogButton {
            id: setResolutionButton
            Layout.fillWidth: true

            // Enabling the closing of the form when the exit button is pressed
            onClicked: {
                var component = Qt.createComponent("Resolution.qml")
                if (component.status === Component.Ready)
                {
                    resolutionWindow = component.createObject(wavelengthsItem)
                }
                resolutionWindow.show()
            }

            GridLayout {

                id: resolutionGirdLayout
                anchors.centerIn: parent

                // Emailing the results of the calculations (the plot, data or both)
                CustomLabel {
                    id: resolLabel
                    Layout.column: 0
                    Layout.row: 0
                    anchors.right: resolValueLabel.left
                    horizontalAlignment: Text.AlignRight
                    rightPadding: coreObj.defaultLabelHeight * 0.5
                    text: qsTr("Change resolution:")
                }
                CustomLabel {
                    id: resolValueLabel
                    Layout.column: 1
                    Layout.row: 0
                    color: 'red'
                    text: wlQml.calculatedWlResolCurrUnits
                }

                // Emailing the results of the calculations (the plot, data or both)
                CustomLabel {
                    id: numPointsLabel
                    Layout.column: 0
                    Layout.row: 1
                    anchors.right: numPointsValueLabel.left
                    horizontalAlignment: Text.AlignRight
                    rightPadding: coreObj.defaultLabelHeight * 0.5
                    text: qsTr("Change data points:")
                }
                CustomLabel {
                    id: numPointsValueLabel
                    Layout.column: 1
                    Layout.row: 1
                    color: 'red'
                    text: wlQml.calculatedNumPoints
                }
            }
        }

        // Wavelength units
        CustomComboBox {
            id: unitsComboBox
            Layout.fillWidth: true
            model: [ "Wavelength, nm",  "Wavelength, micron", "Wavenumber, cm-1", "Frequency, THz" ]  // 0 - Wavelength, nm;  1 - Wavelength, micron, 2 - Wavenumber, cm-1;  3 - Frequency, Hz;  ( 4 - Attenuation, Db try to add later, this is not dB but something involving Db, need to check )
            currentIndex: wlQml.units
            onCurrentIndexChanged: wlQml.units = currentIndex
        }
    }
}
