// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


// Class used for HITRAN log in purporses (this class connects the QML with the C++)
#include "authenticateqml.h"

AuthenticateQml::AuthenticateQml(QObject *parent, Hitran *inHitran) : QObject(parent)
{
    // Saving a reference to the server login object
    hitran = inHitran;

    this->m_authenticate = false;
    this->m_storeEmail = true;
    this->m_storePassword = true;
    this->m_storeHapiToken = true;
    //this->m_email = hitran->getEmail();
    //this->m_password = hitran->getPassword();
    //this->m_hapiToken= hitran->getHapiToken();
    this->m_authenticateType = 0;
}

bool AuthenticateQml::authenticate() const { return this->m_authenticate; }
void AuthenticateQml::setAuthenticate(const bool &authenticate)
{
    if (authenticate != m_authenticate) {
        m_authenticate = authenticate;
        emit authenticateChanged();
    }
}

bool AuthenticateQml::storeEmail() const { return this->m_storeEmail; }
void AuthenticateQml::setStoreEmail(const bool &storeEmail)
{
    if (storeEmail != m_storeEmail) {
        m_storeEmail = storeEmail;
        emit storeEmailChanged();
    }
}

bool AuthenticateQml::storePassword() const { return this->m_storePassword; }
void AuthenticateQml::setStorePassword(const bool &storePassword)
{
    if (storePassword != m_storePassword) {
        m_storePassword = storePassword;
        emit storePasswordChanged();
    }
}

bool AuthenticateQml::storeHapiToken() const { return this->m_storeHapiToken; }
void AuthenticateQml::setStoreHapiToken(const bool &storeHapiToken)
{
    if (storeHapiToken != m_storeHapiToken) {
        m_storeHapiToken = storeHapiToken;
        emit storeHapiTokenChanged();
    }
}

QString AuthenticateQml::email() const
{
    return ""; // hitran->getEmail();
}
void AuthenticateQml::setEmail(const QString &email)
{
    if (email != m_email) {
        m_email = email;
        //hitran->setEmail(m_email);
        emit emailChanged();
    }
}

QString AuthenticateQml::password() const
{
    return ""; // hitran->getPassword();
}
void AuthenticateQml::setPassword(const QString &password)
{
    if (password != m_password) {
        m_password = password;
        //hitran->setPassword(m_password);
        emit passwordChanged();
    }
}

QString AuthenticateQml::hapiToken() const { return hitran->getHapiToken(); }
void AuthenticateQml::setHapiToken(const QString &hapiToken)
{
    if (hapiToken != m_hapiToken) {
        m_hapiToken = hapiToken;
        hitran->setHapiToken(m_hapiToken);
        emit hapiTokenChanged();
    }
}

int AuthenticateQml::authenticateType() const { return this->m_authenticateType; }
void AuthenticateQml::setAuthenticateType(const int &authenticateType)
{
    if (authenticateType != m_authenticateType) {
        m_authenticateType = authenticateType;
        emit authenticateTypeChanged();
    }
}
