// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


// Class used for HITRAN log in purporses (this class connects the QML with the C++)
#ifndef AUTHENTICATEQML_H
#define AUTHENTICATEQML_H

#include <QObject>
#include "hitran.h"

class AuthenticateQml : public QObject
{
    Q_OBJECT

    Q_PROPERTY(bool authenticate READ authenticate WRITE setAuthenticate NOTIFY authenticateChanged)
    Q_PROPERTY(bool storeEmail READ storeEmail WRITE setStoreEmail NOTIFY storeEmailChanged)
    Q_PROPERTY(bool storePassword READ storePassword WRITE setStorePassword NOTIFY storePasswordChanged)
    Q_PROPERTY(bool storeHapiToken READ storeHapiToken WRITE setStoreHapiToken NOTIFY storeHapiTokenChanged)
    Q_PROPERTY(QString email READ email WRITE setEmail NOTIFY emailChanged)
    Q_PROPERTY(QString password READ password WRITE setPassword NOTIFY passwordChanged)
    Q_PROPERTY(QString hapiToken READ hapiToken WRITE setHapiToken NOTIFY hapiTokenChanged)
    Q_PROPERTY(int authenticateType READ authenticateType WRITE setAuthenticateType NOTIFY authenticateTypeChanged)

public:
    explicit AuthenticateQml(QObject *parent = nullptr, Hitran *inHitran = nullptr);

    bool authenticate() const;
    void setAuthenticate(const bool &authenticate);

    bool storeEmail() const;
    void setStoreEmail(const bool &storeEmail);

    bool storePassword() const;
    void setStorePassword(const bool &storePassword);

    bool storeHapiToken() const;
    void setStoreHapiToken(const bool &storeHapiToken);

    QString email() const;
    void setEmail(const QString &email);

    QString password() const;
    void setPassword(const QString &password);

    QString hapiToken() const;
    void setHapiToken(const QString &hapiToken);

    int authenticateType() const;
    void setAuthenticateType(const int &authenticateType);

signals:

    void authenticateChanged();
    void storeEmailChanged();
    void storePasswordChanged();
    void storeHapiTokenChanged();
    void emailChanged();
    void passwordChanged();
    void hapiTokenChanged();
    void authenticateTypeChanged();

public slots:

private:

    Hitran *hitran;  // Object to access and log in to the HITRANonline server

    bool m_authenticate;    // Enable or disable the authentication
    bool m_storeEmail;      // Enable or disable the storing of an email
    bool m_storePassword;   // Enable or disable the stroing of the password
    bool m_storeHapiToken;  // Enable or disable the stroing of the HAPI token
    QString m_email;        // Email to be used for accessing the HITRANonline server
    QString m_password;     // Password to be used with the email for authenticatio at HITRANonline
    QString m_hapiToken;    // HAPI token to access the HITRANonline with authentication
    int m_authenticateType; // Type of authentication (Email and password vs. HAPI token)

};

#endif // AUTHENTICATEQML_H
