// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


// This class is used to determine base folder and file paths for data store and access (i.e. HITRAN database downloads etc.)
#ifndef BASEPATHS_H
#define BASEPATHS_H

#include <QObject>

class BasePaths : public QObject
{
    Q_OBJECT
public:
    explicit BasePaths(QObject *parent = nullptr,
                       bool newExternalStorageAccessGranted = false,                       
                       QString newSdCardBaseDownloadFolderPath = "");

    // Folder with the cumulative HITRAN database file (where the complete HITRAN database file is to be stored), this folder will also contain a sub-folder with the HITRAN downloaded in segments from HITRANOnline
    QString getBaseDownloadFolderPath();
    // This method will return truncated folder paths (initial path removed up to the folders typically visible to the user)
    QString getTruncatedBaseDownloadFolderPath();

    // Folder to store the calculation results (images and text files)
    QString getResultsFolderPath();

    // Internal application folder (typically used to store the downloaded TIPS files, the molparam.txt, and the globalid.txt)
    QString getInternalApplicaationFolderPath();

    // Folder on the SD card with the cumulative HITRAN database file (where the complete HITRAN database file is to be stored), this folder will also contain a sub-folder with the HITRAN downloaded in segments from HITRANOnline
    QString getSdCardBaseDownloadFolderPath();
    void setSdCardBaseDownloadFolderPath(QString newSdCardBaseDownloadFolderPath);

    // Variable is indicating if the internal storage or the SD card should be used to store the downloaded HITRAN database and to store the configuration file
    bool getUseSdCard();
    void setUseSdCard(bool useSdCard);
    // Methods related to the variable indicating if the external storage access was granted (Android only)
    bool getExternalStorageAccessGranted();
    void setExternalStorageAccessGranted(bool externalStorageAccessGranted);

    // Method is used to create directories if they do not exist
    void createDirsIfNoneExist();

    // Methods to save and load the state of if the SD card is to be used instead of the internal storage
    void saveSdCardUseStateToFile();
    void readSdCardUseStateFromFile();

signals:

public slots:

private:

    bool useSdCard;  // Variable is indicating if the SD card is to be used instead of the internal storage
    bool externalStorageAccessGranted;     // Variable is indicating if the external storage access permission has been granted
    QString sDCardBaseDownloadFolderPath;  // Path to the download locatoin on the SD card

    QString applicationPath;  // Path to the internal application data folder
    QString downloadsPath;    // Path to the public downloads folder
    QString documentsPath;    // Path to the public documents folder

};

#endif // BASEPATHS_H
