# bytran -|- spectral calculations for portable devices using the HITRAN database
#
# Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
# www.bytran.org | bytran@bytran.org
# Copyright (c) 2016 Dzianis Pliutau
# Distributed under the MIT license
#
# The MIT License (MIT)
# Copyright (c) 2016 Dzianis Pliutau
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
# modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
# is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
# IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# Serial port is not supported under some platforms (I think WinRT does not support serial port, need to exclude)
QT += qml quick widgets network sensors bluetooth websockets # webview

# Under the universal windows platform and iOS the Serial Port is not defined
# Also the web engine widgets are not defined for mobile platforms (probably need to replace with web-view QML or try to in the future)
!winrt {
    !ios {
        QT += serialport
        !android {
            # QT += webenginewidgets - Maybe will be added later for web-based HITRAN or rother database access
        }
    }
}

android
{
    android: QT += androidextras
    ANDROID_PACKAGE_SOURCE_DIR = $$PWD/android
}

# These libraries are needed for the secure socket layer connection to the HITRAN server
contains(ANDROID_TARGET_ARCH,armeabi-v7a) {
    ANDROID_EXTRA_LIBS = \
        $$PWD/android/libs/openssl/libcrypto.so \
        $$PWD/android/libs/openssl/libssl.so
}

# This is for the Raspberry Pi Sense HAT (it works on Raspberry pi 2 and 3)
linux-rasp-pi3-g++ { QT += sensehat }
linux-rasp-pi2-g++ { QT += sensehat }
linux-rasp-pi3-vc4-g++ { QT += sensehat }

CONFIG += c++11

RESOURCES += qml.qrc

# Additional import path used to resolve QML modules in Qt Creator's code model
QML_IMPORT_PATH =

# Default rules for deployment.
qnx: target.path = /tmp/$${TARGET}/bin
else: unix:!android: target.path = /opt/$${TARGET}/bin
!isEmpty(target.path): INSTALLS += target

RC_ICONS = bytran.ico
ICON = bytran.icns  # the iOS specific code should be commented out for this icon to be included in the desktop, couldn't find a better solution for now

SOURCES += main.cpp \
    atmosphere.cpp \
    auxiliary.cpp \
    bytrancore.cpp \
    colorplot.cpp \
    concpressunitsconvert.cpp \
    configinputoutput.cpp \
    const.cpp \
    custommessagebox.cpp \
    downloadregionsfinder.cpp \
    filedownloader.cpp \
    hitrandownloadoptionsqml.cpp \
    hitraninputoutput.cpp \
    hitranoneiso.cpp \
    hitranonemol.cpp \
    hitranonerec.cpp \
    instrumentfunction.cpp \
    lbl.cpp \
    mollistviewmodel.cpp \
    mollistviewreducedmodel.cpp \
    oneatmmodel.cpp \
    oneisospectra.cpp \
    onelayerspectra.cpp \
    onemolspectra.cpp \
    oneslantlayer.cpp \
    params.cpp \
    pathlengthunitsconvert.cpp \
    permissions.cpp \
    plot.cpp \
    radiance.cpp \
    resultssaver.cpp \
    slant.cpp \
    spectrum.cpp \
    temperatureunitsconvert.cpp \
    tipsfilesdownloader.cpp \
    wavelength.cpp \
    yaxis.cpp \
    humidityconvert.cpp \
    sensors.cpp \
    winphone/sharewindows.cpp \
    winphone/winrtshare.cpp \
    winphone/CppUIThreadComponent.cpp \
    winphone/pch.cpp \
    winphone/UIDispatchActivity.cpp \
    winphone/winrtsharetest.cpp \
    bluetoothlistviewmodel.cpp \
    sensortag.cpp \
    sensorscellphone.cpp \
    sensorstemppresshumid.cpp \
    aboutqml.cpp \
    bluetoothlowenergy.cpp \
    yaxisqml.cpp \
    wavelengthqml.cpp \
    totalpressqml.cpp \
    temperatureqml.cpp \
    spectraresultsqml.cpp \
    shareresultsqml.cpp \
    sensorsqml.cpp \
    pathlengthqml.cpp \
    notifierqml.cpp \
    lineoptionsqml.cpp \
    atmospherelistviewmodel.cpp \
    instrumentfunctionqml.cpp \
    remotecontrol.cpp \
    serialport.cpp \
    serialportqml.cpp \
    remotecontrolqml.cpp \
    serialportslistmodel.cpp \
    websockets.cpp \
    chatclient.cpp \
    chatserver.cpp \
    bluetoothspp.cpp \
    bluetoothdiscoverdevices.cpp \
    auxisotopesarray.cpp \
    sensehat.cpp \
    authenticateqml.cpp \
    hitrandownloadregionsfinder.cpp \
    hitran.cpp \
    hitranauxiliary.cpp \
    hitrandownloadhapi.cpp \
    lblprecalc.cpp \
    lblprecalconeiso.cpp \
    lblprecalconemol.cpp \
    basepaths.cpp

HEADERS += \
    atmosphere.h \
    auxiliary.h \
    bytrancore.h \
    colorplot.h \
    concpressunitsconvert.h \
    configinputoutput.h \
    const.h \
    custommessagebox.h \
    downloadregionsfinder.h \
    filedownloader.h \
    hitraninputoutput.h \
    hitranoneiso.h \
    hitranonemol.h \
    hitranonerec.h \
    instrumentfunction.h \
    lbl.h \
    mollistviewmodel.h \
    mollistviewreducedmodel.h \
    oneatmmodel.h \
    oneisospectra.h \
    onelayerspectra.h \
    onemolspectra.h \
    oneslantlayer.h \
    params.h \
    pathlengthunitsconvert.h \
    permissions.h \
    plot.h \
    radiance.h \
    resultssaver.h \
    slant.h \
    spectrum.h \
    temperatureunitsconvert.h \
    tipsfilesdownloader.h \
    wavelength.h \
    yaxis.h \
    humidityconvert.h \
    sensors.h \
    winphone/sharewindows.h \
    winphone/winrtshare.h \
    winphone/CppUIThreadComponent.h \
    winphone/pch.h \
    winphone/UIDispatchActivity.h \
    winphone/winrtsharetest.h \
    bluetoothlistviewmodel.h \
    sensortag.h \
    sensorscellphone.h \
    sensorstemppresshumid.h \
    aboutqml.h \
    bluetoothlowenergy.h \
    yaxisqml.h \
    wavelengthqml.h \
    totalpressqml.h \
    temperatureqml.h \
    spectraresultsqml.h \
    shareresultsqml.h \
    sensorsqml.h \
    pathlengthqml.h \
    notifierqml.h \
    lineoptionsqml.h \
    instrumentfunctionqml.h \
    hitrandownloadoptionsqml.h \
    atmospherelistviewmodel.h \
    remotecontrol.h \
    serialport.h \
    serialportqml.h \
    remotecontrolqml.h \
    serialportslistmodel.h \
    websockets.h \
    chatclient.h \
    chatserver.h \
    bluetoothspp.h \
    bluetoothdiscoverdevices.h \
    auxisotopesarray.h \
    sensehat.h \
    authenticateqml.h \
    hitrandownloadregionsfinder.h \
    hitran.h \
    hitranauxiliary.h \
    hitrandownloadhapi.h \
    lblprecalc.h \
    lblprecalconeiso.h \
    lblprecalconemol.h \
    basepaths.h

DISTFILES += \
    android/AndroidManifest.xml \
    android/gradle/wrapper/gradle-wrapper.jar \
    android/gradlew \
    android/res/values/libs.xml \
    android/build.gradle \
    android/gradle/wrapper/gradle-wrapper.properties \
    android/gradlew.bat \
    android/src/org/bytran/ShareIntent.java \
    android/src/org/bytran/Permissions.java \
    android/src/org/bytran/ShowPermissionRationale.java

# These are for the iOS operating system to be able to share images and data
OBJECTIVE_SOURCES += ios/iosshare.mm \
    ios/iosmessagedialog.mm \
    ios/iosdefaulttextsize.mm
HEADERS += ios/iosshare.h \
    ios/iosmessagedialog.h \
    ios/iosdefaulttextsize.h

# ios icon sets etc. (when compiled for Mac the first 2 paragraphs of code should not be there as it interferes with the Mac icon)
ios
{
    QMAKE_INFO_PLIST = ios/Info.plist
    ios_icon.files = $$files($$PWD/ios-iconset/red-background/Icon*.png)
    QMAKE_BUNDLE_DATA += ios_icon

    app_launch_images.files = $$PWD/ios-launchimages/StartScreen.xib $$files($$PWD/ios-launchimages/LaunchImage*.png) $$PWD/images/bytran136.png
    QMAKE_BUNDLE_DATA += app_launch_images

    # This change was made in bytran 1.2 due to the new requirements to place icons in an asset catalog file
    QMAKE_ASSET_CATALOGS += ios-xcassets/Media.xcassets

    # This segment should be there for Mac as well
    Q_ENABLE_BITCODE.name = ENABLE_BITCODE
    Q_ENABLE_BITCODE.value = NO
    QMAKE_MAC_XCODE_SETTINGS += Q_ENABLE_BITCODE
}

# The approach with the slashes was used to make this work - https://bugreports.qt.io/browse/QTBUG-43468
# There icons are not used any more at compile time, instead the precompiled project (with the -tp vc option)
# is opened in visual studio and the icons are automatically generated there, then replaced with the ones in the
# winphone-icons bytran source code folder
winphone {
    WINRT_MANIFEST = $$PWD/winphone/AppxManifest.xml
    WINRT_MANIFEST.logo_44x44 = $$PWD/winphone8-icons/white-on-red/logo_44x44.png
    WINRT_MANIFEST.logo_71x71 = $$PWD/winphone8-icons/white-on-red/logo_71x71.png
    WINRT_MANIFEST.logo_150x150 = $$PWD/winphone8-icons/white-on-red/logo_150x150.png
    WINRT_MANIFEST.logo_310x150 = $$PWD/winphone8-icons/white-on-red/logo_310x150.png
    WINRT_MANIFEST.logo_480x800 = $$PWD/winphone8-icons/white-on-red/logo_480x800.png
    WINRT_MANIFEST.logo_store = $$PWD/winphone8-icons/white-on-red/logo_store.png
    #QMAKE_CXXFLAGS += /ZW
    QMAKE_CXXFLAGS += /ZW /AI\"C:\Program Files (x86)\Microsoft Visual Studio 14.0\VC\vcpackages\"
}
