// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


// This object is used to download auxiliary HITRAN data (molparam.txt, TIPS files and the Global ids file)
#ifndef HITRANAUXILIARY_H
#define HITRANAUXILIARY_H

#include <QObject>
#include "tipsfilesdownloader.h"

class HitranAuxiliary : public QObject
{
    Q_OBJECT
public:

    explicit HitranAuxiliary(QObject *parent = nullptr);

    // Methods to get and set the paths to the folders where HITRAN, molparam.txt, and the TIPS files are stored
    QString getMolparamFolder();
    void setMolparamFolder(QString newMolparamFolder);

    // Paths to the online download folders with the molparam and the TIPS files
    QString getMolparamDownloadAddressPath();
    void setMolparamDownloadAddressPath(QString newMolparamDownloadAddressPath);
    // Paths to the online download folders with the molparam and the TIPS files
    QString getGloabalIdTxtDownloadAddressPath();
    void setGlobalIdTxtDownloadAddressPath(QString newGlobalIdTxtDownloadAddressPath);

    // Method to download the molparam.txt file
    void downloadMolparamTxt();
    // Method to download the custom file with the isotopes and their corresponding global ID number
    void downloadGlobalIdTxt();
    // Method to download files with the total internal partition sums
    void downloadTipsFiles();

    // Method to get the TIPS object
    TipsFilesDownloader *getTipsDownloader();

signals:

    // Signal used to send notifications to the user of what is going on in the program
    void notify(QString message, double value, bool visible, bool cancelButtonVisible);

    // Signal is emited when molparam.txt download is complete
    void molparamTxtDownloadComplete();
    // Signal is emitted once the global ids file has been downloaded
    void globalIdTxtDownloadComplete();
    // Signal is emited when the TIPS data download is complete
    void tipsDownloadComplete();

    // Singnal is emited to open the ok notification dialog
    void openOkDialog(QString text);

public slots:

    // Slot is called after the molparam file has been downloaded
    void molparamOrGlobalidDownloaded(bool success);

    // Slot is called once the TIPS files have finished downloading
    void tipsFilesDownloaded(bool success);

private:

    // Local folders to the downloaded HITRAN auxiliary data files
    QString molparamFolder;  // Path to the folder with the molparam.txt file
    QString tipsFolder;      // Path to the folder with the TIPS data

    // Paths to the online download folders with the molparam and the TIPS files
    QString molparamDownloadAddressPath;
    QString globalIdTxtDownloadAddressPath;
    QString tipsDownloadAddressPath;

    // File downloader object to download the TIPS files and the file with the molecular information
    FileDownloader *fileDownloader;

    // Object to download TIPS files
    TipsFilesDownloader *tipsDownloader;

};

#endif // HITRANAUXILIARY_H
