// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


// Object for one isotope to contain selected pre-calculated line-by-line variables
#ifndef LBLPRECALCONEISO_H
#define LBLPRECALCONEISO_H

class LblPreCalcOneIso
{

public:

    explicit LblPreCalcOneIso();

    // Current to terrestrial abundance ratio
    double getAbundRatio();
    void setAbundRatio(double newAbundRatio);

    // TIPS ratio (ratio of TIPS at current temperature to the TIPS at 296K)
    double getTipsRatio();
    void setTipsRatio(double newTipsRatio);

    // Pre-calculated calues for the Voigt profile
    double getVoigtConst1();
    void setVoigtConst1(double newVoigtConst1);

private:

    double abundRatio;  // Ratio of the isotope abundance in the current atmosphere to the terrestrial abundance
    double tipsRatio;   // TIPS ratio (ratio of TIPS at the current temperature to the TIPS at 296K)

    double voigtConst1;   // Voigt constant for line-by-line calculations
};

#endif // LBLPRECALCONEISO_H
