// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// Class to contain calcualted spectra for a single layer (currently the molecules and composite spectra), the isotopes and separate lines may be added later if need be
#ifndef ONELAYERSPECTRA_H
#define ONELAYERSPECTRA_H

#include "spectrum.h"
#include "onemolspectra.h"
#include <QVector>

class OneLayerSpectra
{
public:

    OneLayerSpectra();

    // Methods to get the unprocessed and the instrument function processed cross-sectional spectra
    Spectrum *getCrossSect();
    Spectrum *getInstrFunctCrossSect();

    // Methods to get and set the composite spectrum (unprocessed and that processed with the instrument function)
    Spectrum *getOptDepth();
    Spectrum *getInstrFunctOptDepth();

    // Method to get the plain (unprocessed and that processed with the instrument function radiance spectruum)
    Spectrum *getPlainRadiance();
    Spectrum *getInstrFunctRadiance();

    // Methods to get and set the molecular attenuation spectra
    QVector<OneMolSpectra> *getMol();

    // Method to get and set the molecular spectra at a specified index
    OneMolSpectra *getMolAt(int index);

    // Method to return a spectrum of interest (any type of spectrum based on the spectrum index)
    Spectrum *getSpec(int composite, int mol, int iso, long line);

private:

    Spectrum crossSectionSpectrum;  // Total absorption cross-section spectrum
    Spectrum instFunctCrossSectSpectrum; // Instrument function processed absorption cross-section spectrum
    Spectrum compositeSpectrum;  // Spectrum containing the composite for the entire horizontal path
    Spectrum instrFunctSpectrum;   // Spectrum smoothed with the instrument function
    Spectrum plainRadiance;      // Spectrum with the unprocessed radiance
    Spectrum instrFunctRadiance; // Radiance calculation performed on the instrument funciton processed spectrum
    QVector<OneMolSpectra> molSpectra;  // Array of spectra for each individual molecule (molecular totals for each molecule and also includes isotopes etc.)

};

#endif // ONELAYERSPECTRA_H
